(function UMD(name, context, dependencies, definition) {
  if (typeof define === "function" && define.amd) {
    dependencies = Object.keys(dependencies).map(p => p.replace(/^\.\//, ""));
    define(name, dependencies, definition);
  } else if (typeof module !== "undefined" && module.exports) {
    dependencies = Object.keys(dependencies).map(p => require(p));
    module.exports = definition(...dependencies);
  } else {
    dependencies = Object.values(dependencies).map(n => context[n]);
    context[name] = definition(...dependencies);
  }
})("Mz", typeof globalThis != "undefined" ? globalThis : typeof global != "undefined" ? global : typeof window != "undefined" ? window : typeof self != "undefined" ? self : new Function("return this")(), {
  "./umd.js": "UMD",
  "./esm.js": "ESM"
}, function DEF(UMD, ESM) {
  "use strict";

  var buildUMD = UMD;
  var {
    bundle: bundleUMD,
    index: umdIndex
  } = UMD;
  var buildESM = ESM;
  var {
    index: esmIndex
  } = ESM;
  let _exp = {};
  _exp = build;
  _exp.build = build;
  _exp.defaultLibConfig = defaultLibConfig;
  _exp.bundleUMD = bundleUMD;
  _exp.umdIndex = umdIndex;
  _exp.esmIndex = esmIndex;

  function build(config, pathStr, code, depMap = {}) {
    config = defaultLibConfig(config);
    var output = {};

    if (config.buildESM) {
      output.esm = buildESM(config, pathStr, code, depMap);
    }

    if (config.buildUMD) {
      output.umd = buildUMD(config, pathStr, code, depMap);
    }

    return output;
  }

  function defaultLibConfig({
    ignoreUnknownDependency = false,
    ignoreCircularDependency = false,
    ".mjs": renameMJS = false,
    ".cjs": renameCJS = false,
    namespaceImport = false,
    namespaceExport = false,
    exportDefaultFrom = false,
    ...otherConfig
  } = {}) {
    return {
      ignoreUnknownDependency,
      ignoreCircularDependency,
      ".mjs": renameMJS,
      ".cjs": renameCJS,
      namespaceImport,
      namespaceExport,
      exportDefaultFrom,
      ...otherConfig
    };
  }

  return _exp;
});